#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" This package contains the information classes that support distribution management in general.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61970.Core import IdentifiedObject
from CIM import Element
from CIM.IEC61970.Domain import AbsoluteDate
from CIM.IEC61970.Domain import Seconds
from CIM.IEC61970.Domain import StringQuantity



from enthought.traits.api import Instance, List, Property, Str, Date, Int, Bool
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "Organisation" class:
#------------------------------------------------------------------------------

class Organisation(IdentifiedObject):
    """ Organisation that might have roles as utility, contractor, supplier, manufacturer, customer, etc.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    BusinessRoles = List(Instance("CIM.IEC61968.Informative.InfCommon.BusinessRole"))

    # All telephone numbers of this organisation.
    TelephoneNumbers = List(Instance("CIM.IEC61968.Common.TelephoneNumber"),
        desc="All telephone numbers of this organisation.")

    # Street address.
    streetAddress = Instance("CIM.IEC61968.Common.StreetAddress",
        desc="Street address.",
        transient=True,
        editor=InstanceEditor(name="_streetaddresss"))

    def _get_streetaddresss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.StreetAddress" ]
        else:
            return []

    _streetaddresss = Property(fget=_get_streetaddresss)

    MarketRoles = List(Instance("CIM.IEC61968.Informative.InfCommon.MarketRole"))

    # Postal address, potentially different than 'streetAddress' (e.g., another city).
    postalAddress = Instance("CIM.IEC61968.Common.PostalAddress",
        desc="Postal address, potentially different than 'streetAddress' (e.g., another city).",
        transient=True,
        editor=InstanceEditor(name="_postaladdresss"))

    def _get_postaladdresss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.PostalAddress" ]
        else:
            return []

    _postaladdresss = Property(fget=_get_postaladdresss)

    # All electronic addresses of this organisation.
    ElectronicAddresses = List(Instance("CIM.IEC61968.Common.ElectronicAddress"),
        desc="All electronic addresses of this organisation.")

    #--------------------------------------------------------------------------
    #  Begin "Organisation" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "BusinessRoles", "TelephoneNumbers", "streetAddress", "MarketRoles", "postalAddress", "ElectronicAddresses",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.Organisation",
        title="Organisation",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Organisation" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ActivityRecord" class:
#------------------------------------------------------------------------------

class ActivityRecord(IdentifiedObject):
    """ Records activity for an entity at a point in time; activity may be for an event that has already occurred or for a planned activity.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    MarketFactors = List(Instance("CIM.IEC61968.Informative.MarketOperations.MarketFactors"))

    # All documents for which this activity record has been created.
    Documents = List(Instance("CIM.IEC61968.Common.Document"),
        desc="All documents for which this activity record has been created.")

    Organisations = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpOrganisation"))

    ScheduledEvent = Instance("CIM.IEC61968.Informative.InfCommon.ScheduledEvent",
        transient=True,
        opposite="ActivityRecord",
        editor=InstanceEditor(name="_scheduledevents"))

    def _get_scheduledevents(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfCommon.ScheduledEvent" ]
        else:
            return []

    _scheduledevents = Property(fget=_get_scheduledevents)

    # All assets for which this activity record has been created.
    Assets = List(Instance("CIM.IEC61968.Assets.Asset"),
        desc="All assets for which this activity record has been created.")

    ErpPersons = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpPerson"))

    Locations = List(Instance("CIM.IEC61968.Common.Location"))

    # Information on consequence of event resulting in this activity record.
    status = Instance("CIM.IEC61968.Common.Status",
        desc="Information on consequence of event resulting in this activity record.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    # Reason for event resulting in this activity record, typically supplied when user initiated.
    reason = Str(desc="Reason for event resulting in this activity record, typically supplied when user initiated.")

    # Category of event resulting in this activity record.
    category = Str(desc="Category of event resulting in this activity record.")

    # Severity level of event resulting in this activity record.
    severity = Str(desc="Severity level of event resulting in this activity record.")

    # Date and time this activity record has been created (different from the 'status.dateTime', which is the time of a status change of the associated object, if applicable).
    createdDateTime = Date(desc="Date and time this activity record has been created (different from the 'status.dateTime', which is the time of a status change of the associated object, if applicable).")

    #--------------------------------------------------------------------------
    #  Begin "ActivityRecord" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "reason", "category", "severity", "createdDateTime",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "MarketFactors", "Documents", "Organisations", "ScheduledEvent", "Assets", "ErpPersons", "Locations", "status",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.ActivityRecord",
        title="ActivityRecord",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ActivityRecord" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Document" class:
#------------------------------------------------------------------------------

class Document(IdentifiedObject):
    """ Parent class for different groupings of information collected and managed as a part of a business process. It will frequently contain references to other objects, such as assets, people and power system resources.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All activity records created for this document.
    ActivityRecords = List(Instance("CIM.IEC61968.Common.ActivityRecord"),
        desc="All activity records created for this document.")

    ErpOrganisationRoles = List(Instance("CIM.IEC61968.Informative.InfERPSupport.DocOrgRole"))

    ScheduledEvents = List(Instance("CIM.IEC61968.Informative.InfCommon.ScheduledEvent"))

    FromDocumentRoles = List(Instance("CIM.IEC61968.Informative.InfCommon.DocDocRole"))

    LocationRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.DocLocRole"))

    PowerSystemResourceRoles = List(Instance("CIM.IEC61968.Informative.InfCommon.DocPsrRole"))

    NetworkDataSets = List(Instance("CIM.IEC61968.Informative.InfOperations.NetworkDataSet"))

    ErpPersonRoles = List(Instance("CIM.IEC61968.Informative.InfERPSupport.DocErpPersonRole"))

    ChangeItems = List(Instance("CIM.IEC61968.Informative.InfOperations.ChangeItem"))

    # Measurements are specified in types of documents, such as procedures.
    Measurements = List(Instance("CIM.IEC61970.Meas.Measurement"),
        desc="Measurements are specified in types of documents, such as procedures.")

    # Status of this document. For status of subject matter this document represents (e.g., Agreement, Work), use 'status' attribute. Example values for 'docStatus.status' are draft, approved, cancelled, etc.
    docStatus = Instance("CIM.IEC61968.Common.Status",
        desc="Status of this document. For status of subject matter this document represents (e.g., Agreement, Work), use 'status' attribute. Example values for 'docStatus.status' are draft, approved, cancelled, etc.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    ScheduleParameterInfos = List(Instance("CIM.IEC61968.Informative.InfCommon.ScheduleParameterInfo"))

    ElectronicAddress = Instance("CIM.IEC61968.Common.ElectronicAddress",
        transient=True,
        opposite="Document",
        editor=InstanceEditor(name="_electronicaddresss"))

    def _get_electronicaddresss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.ElectronicAddress" ]
        else:
            return []

    _electronicaddresss = Property(fget=_get_electronicaddresss)

    ToDocumentRoles = List(Instance("CIM.IEC61968.Informative.InfCommon.DocDocRole"))

    # Status of subject matter (e.g., Agreement, Work) this document represents. For status of the document itself, use 'docStatus' attribute.
    status = Instance("CIM.IEC61968.Common.Status",
        desc="Status of subject matter (e.g., Agreement, Work) this document represents. For status of the document itself, use 'docStatus' attribute.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    AssetRoles = List(Instance("CIM.IEC61968.Informative.InfAssets.DocAssetRole"))

    ChangeSets = List(Instance("CIM.IEC61968.Informative.InfOperations.ChangeSet"))

    # Document subject.
    subject = Str(desc="Document subject.")

    # Revision number for this document.
    revisionNumber = Str(desc="Revision number for this document.")

    # Utility-specific categorisation of this document, according to their corporate standards, practices, and existing IT systems (e.g., for management of assets, maintenance, work, outage, customers, etc.).
    category = Str(desc="Utility-specific categorisation of this document, according to their corporate standards, practices, and existing IT systems (e.g., for management of assets, maintenance, work, outage, customers, etc.).")

    # Date and time this document was last modified. Documents may potentially be modified many times during their lifetime.
    lastModifiedDateTime = Date(desc="Date and time this document was last modified. Documents may potentially be modified many times during their lifetime.")

    # Document title.
    title = Str(desc="Document title.")

    # Date and time that this document was created.
    createdDateTime = Date(desc="Date and time that this document was created.")

    #--------------------------------------------------------------------------
    #  Begin "Document" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets",
                label="References", columns=1),
            dock="tab"),
        id="CIM.IEC61968.Common.Document",
        title="Document",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Document" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PositionPoint" class:
#------------------------------------------------------------------------------

class PositionPoint(Element):
    """ Set of spatial coordinates that determine a point. A sequence of PositionPoints can be used to describe: - physical location of non-point oriented objects like cables or lines, or - area of an object like a substation, a geographical zone or a diagram object.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Location that this position point describes.
    Location = Instance("CIM.IEC61968.Common.Location",
        desc="Location that this position point describes.",
        transient=True,
        opposite="PositionPoints",
        editor=InstanceEditor(name="_locations"))

    def _get_locations(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Location" ]
        else:
            return []

    _locations = Property(fget=_get_locations)

    # Zero-relative sequence number of this point within a series of points.
    sequenceNumber = Int(desc="Zero-relative sequence number of this point within a series of points.")

    # (if applicable) Z axis position.
    zPosition = Str(desc="(if applicable) Z axis position.")

    # X axis position.
    xPosition = Str(desc="X axis position.")

    # Y axis position.
    yPosition = Str(desc="Y axis position.")

    #--------------------------------------------------------------------------
    #  Begin "PositionPoint" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "sequenceNumber", "zPosition", "xPosition", "yPosition",
                label="Attributes"),
            VGroup("Parent", "Location",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.PositionPoint",
        title="PositionPoint",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PositionPoint" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Location" class:
#------------------------------------------------------------------------------

class Location(IdentifiedObject):
    """ The place, scene, or point of something where someone or something has been, is, and/or will be at a given moment in time. It may be: - Spatial location of an actual or planned structure, or a set of point-oriented structures (as a substation, structure, building, town, etc.) or diagram objects, which may be defined as a point or polygon, or, - Path of an underground or overhead conductor, or a linear diagram object.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    DocumentRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.DocLocRole"))

    ErpPersonRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.ErpPersonLocRole"))

    # All electronic addresses of this location.
    ElectronicAddresses = List(Instance("CIM.IEC61968.Common.ElectronicAddress"),
        desc="All electronic addresses of this location.")

    ChangeItems = List(Instance("CIM.IEC61968.Informative.InfOperations.ChangeItem"))

    Routes = List(Instance("CIM.IEC61968.Informative.InfLocations.Route"))

    # Sequence of position points describing this location.
    PositionPoints = List(Instance("CIM.IEC61968.Common.PositionPoint"),
        desc="Sequence of position points describing this location.")

    GmlSelectors = List(Instance("CIM.IEC61968.Informative.InfGMLSupport.GmlSelector"))

    # Main address of the location.
    mainAddress = Instance("CIM.IEC61968.Common.StreetAddress",
        desc="Main address of the location.",
        transient=True,
        editor=InstanceEditor(name="_streetaddresss"))

    def _get_streetaddresss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.StreetAddress" ]
        else:
            return []

    _streetaddresss = Property(fget=_get_streetaddresss)

    FromLocationRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.LocLocRole"))

    # Status of this location.
    status = Instance("CIM.IEC61968.Common.Status",
        desc="Status of this location.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    ToLocationRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.LocLocRole"))

    # All telephone numbers of this location.
    TelephoneNumbers = List(Instance("CIM.IEC61968.Common.TelephoneNumber"),
        desc="All telephone numbers of this location.")

    # Secondary address of the location. For example, PO Box address may have different ZIP code than that in the 'mainAddress'.
    secondaryAddress = Instance("CIM.IEC61968.Common.StreetAddress",
        desc="Secondary address of the location. For example, PO Box address may have different ZIP code than that in the 'mainAddress'.",
        transient=True,
        editor=InstanceEditor(name="_streetaddresss"))

    def _get_streetaddresss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.StreetAddress" ]
        else:
            return []

    _streetaddresss = Property(fget=_get_streetaddresss)

    LandProperties = List(Instance("CIM.IEC61968.Informative.InfLocations.LandProperty"))

    Measurements = List(Instance("CIM.IEC61970.Meas.Measurement"))

    ErpOrganisationRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.OrgLocRole"))

    DimensionsInfo = Instance("CIM.IEC61968.Informative.InfAssets.DimensionsInfo",
        transient=True,
        opposite="Locations",
        editor=InstanceEditor(name="_dimensionsinfos"))

    def _get_dimensionsinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.DimensionsInfo" ]
        else:
            return []

    _dimensionsinfos = Property(fget=_get_dimensionsinfos)

    AssetRoles = List(Instance("CIM.IEC61968.Informative.InfLocations.AssetLocRole"))

    Crews = List(Instance("CIM.IEC61968.Informative.InfWork.Crew"))

    RedLines = List(Instance("CIM.IEC61968.Informative.InfLocations.RedLine"))

    GmlObservatins = List(Instance("CIM.IEC61968.Informative.InfGMLSupport.GmlObservation"))

    Hazards = List(Instance("CIM.IEC61968.Informative.InfLocations.Hazard"))

    ActivityRecords = List(Instance("CIM.IEC61968.Common.ActivityRecord"))

    # Utility-specific code for the location.
    corporateCode = Str(desc="Utility-specific code for the location.")

    # (if applicable) Direction that allows field crews to quickly find a given asset. For a given location, such as a street address, this is the relative direction in which to find the asset. For example, a Streetlight may be located at the 'NW' (northwest) corner of the customer's site, or a ServiceDeliveryPoint may be located on the second floor of an apartment building.
    direction = Str(desc="(if applicable) Direction that allows field crews to quickly find a given asset. For a given location, such as a street address, this is the relative direction in which to find the asset. For example, a Streetlight may be located at the 'NW' (northwest) corner of the customer's site, or a ServiceDeliveryPoint may be located on the second floor of an apartment building.")

    # True if the first and last point in the sequence of associated PositionPoints are to be connected, thus forming a polygon rather than merely a sequence of line segments.
    isPolygon = Bool(desc="True if the first and last point in the sequence of associated PositionPoints are to be connected, thus forming a polygon rather than merely a sequence of line segments.")

    # Category by utility's corporate standards and practices, relative to the location itself (e.g., geographical, functional accounting, etc., not a given property that happens to exist at that location).
    category = Str(desc="Category by utility's corporate standards and practices, relative to the location itself (e.g., geographical, functional accounting, etc., not a given property that happens to exist at that location).")

    # (if applicable) Reference to geographical information source, often external to the utility.
    geoInfoReference = Str(desc="(if applicable) Reference to geographical information source, often external to the utility.")

    #--------------------------------------------------------------------------
    #  Begin "Location" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "corporateCode", "direction", "isPolygon", "category", "geoInfoReference",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "DocumentRoles", "ErpPersonRoles", "ElectronicAddresses", "ChangeItems", "Routes", "PositionPoints", "GmlSelectors", "mainAddress", "FromLocationRoles", "status", "ToLocationRoles", "TelephoneNumbers", "secondaryAddress", "LandProperties", "Measurements", "ErpOrganisationRoles", "DimensionsInfo", "AssetRoles", "Crews", "RedLines", "GmlObservatins", "Hazards", "ActivityRecords",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Common.Location",
        title="Location",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Location" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "StreetAddress" class:
#------------------------------------------------------------------------------

class StreetAddress(Element):
    """ General purpose street address information.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Status of this address.
    status = Instance("CIM.IEC61968.Common.Status",
        desc="Status of this address.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    # Town detail.
    townDetail = Instance("CIM.IEC61968.Common.TownDetail",
        desc="Town detail.",
        transient=True,
        editor=InstanceEditor(name="_towndetails"))

    def _get_towndetails(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.TownDetail" ]
        else:
            return []

    _towndetails = Property(fget=_get_towndetails)

    # Street detail.
    streetDetail = Instance("CIM.IEC61968.Common.StreetDetail",
        desc="Street detail.",
        transient=True,
        editor=InstanceEditor(name="_streetdetails"))

    def _get_streetdetails(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.StreetDetail" ]
        else:
            return []

    _streetdetails = Property(fget=_get_streetdetails)

    #--------------------------------------------------------------------------
    #  Begin "StreetAddress" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID",
                label="Attributes"),
            VGroup("Parent", "status", "townDetail", "streetDetail",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.StreetAddress",
        title="StreetAddress",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StreetAddress" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TelephoneNumber" class:
#------------------------------------------------------------------------------

class TelephoneNumber(IdentifiedObject):
    """ Telephone number.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Organisation owning this telephone number.
    Organisation = Instance("CIM.IEC61968.Common.Organisation",
        desc="Organisation owning this telephone number.",
        transient=True,
        opposite="TelephoneNumbers",
        editor=InstanceEditor(name="_organisations"))

    def _get_organisations(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Organisation" ]
        else:
            return []

    _organisations = Property(fget=_get_organisations)

    # Location owning this telephone number.
    Location = Instance("CIM.IEC61968.Common.Location",
        desc="Location owning this telephone number.",
        transient=True,
        opposite="TelephoneNumbers",
        editor=InstanceEditor(name="_locations"))

    def _get_locations(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Location" ]
        else:
            return []

    _locations = Property(fget=_get_locations)

    # (if applicable) City code.
    cityCode = Str(desc="(if applicable) City code.")

    # Country code.
    countryCode = Str(desc="Country code.")

    # (if applicable) Extension for this telephone number.
    extension = Str(desc="(if applicable) Extension for this telephone number.")

    # Area or region code.
    areaCode = Str(desc="Area or region code.")

    # Main (local) part of this telephone number.
    localNumber = Str(desc="Main (local) part of this telephone number.")

    #--------------------------------------------------------------------------
    #  Begin "TelephoneNumber" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "cityCode", "countryCode", "extension", "areaCode", "localNumber",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "Organisation", "Location",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.TelephoneNumber",
        title="TelephoneNumber",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TelephoneNumber" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DateTimeInterval" class:
#------------------------------------------------------------------------------

class DateTimeInterval(Element):
    """ Interval of date and time.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Date and time that this interval ended.
    end = Date(desc="Date and time that this interval ended.")

    # Date and time that this interval started.
    start = Date(desc="Date and time that this interval started.")

    #--------------------------------------------------------------------------
    #  Begin "DateTimeInterval" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "end", "start",
                label="Attributes"),
            VGroup("Parent",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.DateTimeInterval",
        title="DateTimeInterval",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DateTimeInterval" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PostalAddress" class:
#------------------------------------------------------------------------------

class PostalAddress(Element):
    """ General purpose postal address information.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Street detail.
    streetDetail = Instance("CIM.IEC61968.Common.StreetDetail",
        desc="Street detail.",
        transient=True,
        editor=InstanceEditor(name="_streetdetails"))

    def _get_streetdetails(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.StreetDetail" ]
        else:
            return []

    _streetdetails = Property(fget=_get_streetdetails)

    # Town detail.
    townDetail = Instance("CIM.IEC61968.Common.TownDetail",
        desc="Town detail.",
        transient=True,
        editor=InstanceEditor(name="_towndetails"))

    def _get_towndetails(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.TownDetail" ]
        else:
            return []

    _towndetails = Property(fget=_get_towndetails)

    # Post office box.
    poBox = Str(desc="Post office box.")

    # Postal code for the address.
    postalCode = Str(desc="Postal code for the address.")

    #--------------------------------------------------------------------------
    #  Begin "PostalAddress" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "poBox", "postalCode",
                label="Attributes"),
            VGroup("Parent", "streetDetail", "townDetail",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.PostalAddress",
        title="PostalAddress",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PostalAddress" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TownDetail" class:
#------------------------------------------------------------------------------

class TownDetail(Element):
    """ Town details, in the context of address.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Name of the state or province.
    stateOrProvince = Str(desc="Name of the state or province.")

    # Town code.
    code = Str(desc="Town code.")

    # Town name.
    name = Str(desc="Town name.")

    # Town section. For example, it is common for there to be 36 sections per township.
    section = Str(desc="Town section. For example, it is common for there to be 36 sections per township.")

    # Name of the country.
    country = Str(desc="Name of the country.")

    #--------------------------------------------------------------------------
    #  Begin "TownDetail" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "stateOrProvince", "code", "name", "section", "country",
                label="Attributes"),
            VGroup("Parent",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.TownDetail",
        title="TownDetail",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TownDetail" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ElectronicAddress" class:
#------------------------------------------------------------------------------

class ElectronicAddress(IdentifiedObject):
    """ Electronic address information.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ErpTelephoneNumbers = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpTelephoneNumber"))

    # All locations having this electronic address.
    Locations = List(Instance("CIM.IEC61968.Common.Location"),
        desc="All locations having this electronic address.")

    Cashier = Instance("CIM.IEC61968.PaymentMetering.Cashier",
        transient=True,
        opposite="ElectronicAddresses",
        editor=InstanceEditor(name="_cashiers"))

    def _get_cashiers(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.Cashier" ]
        else:
            return []

    _cashiers = Property(fget=_get_cashiers)

    ErpPerson = Instance("CIM.IEC61968.Informative.InfERPSupport.ErpPerson",
        transient=True,
        opposite="ElectronicAddresses",
        editor=InstanceEditor(name="_erppersons"))

    def _get_erppersons(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfERPSupport.ErpPerson" ]
        else:
            return []

    _erppersons = Property(fget=_get_erppersons)

    # Asset owning this electronic address.
    Asset = Instance("CIM.IEC61968.Assets.Asset",
        desc="Asset owning this electronic address.",
        transient=True,
        opposite="ElectronicAddresses",
        editor=InstanceEditor(name="_assets"))

    def _get_assets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Assets.Asset" ]
        else:
            return []

    _assets = Property(fget=_get_assets)

    # Status of this electronic address.
    status = Instance("CIM.IEC61968.Common.Status",
        desc="Status of this electronic address.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    Document = Instance("CIM.IEC61968.Common.Document",
        transient=True,
        opposite="ElectronicAddress",
        editor=InstanceEditor(name="_documents"))

    def _get_documents(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Document" ]
        else:
            return []

    _documents = Property(fget=_get_documents)

    # Organisation owning this electronic address.
    Organisation = Instance("CIM.IEC61968.Common.Organisation",
        desc="Organisation owning this electronic address.",
        transient=True,
        opposite="ElectronicAddresses",
        editor=InstanceEditor(name="_organisations"))

    def _get_organisations(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Organisation" ]
        else:
            return []

    _organisations = Property(fget=_get_organisations)

    # World Wide Web address.
    web = Str(desc="World Wide Web address.")

    # Email address.
    email = Str(desc="Email address.")

    # Password needed to log in.
    password = Str(desc="Password needed to log in.")

    # Address on local area network.
    lan = Str(desc="Address on local area network.")

    # User ID needed to log in, which can be for an individual person, an organisation, a location, etc.
    userID = Str(desc="User ID needed to log in, which can be for an individual person, an organisation, a location, etc.")

    # Radio address.
    radio = Str(desc="Radio address.")

    #--------------------------------------------------------------------------
    #  Begin "ElectronicAddress" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "web", "email", "password", "lan", "userID", "radio",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ErpTelephoneNumbers", "Locations", "Cashier", "ErpPerson", "Asset", "status", "Document", "Organisation",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.ElectronicAddress",
        title="ElectronicAddress",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ElectronicAddress" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TimePoint" class:
#------------------------------------------------------------------------------

class TimePoint(IdentifiedObject):
    """ A point in time within a sequence of points in time relative to a TimeSchedule.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Interval defining the window of time that this time point is valid (for example, seasonal, only on weekends, not on weekends, only 8:00 to 5:00, etc.).
    window = Instance("CIM.IEC61968.Common.DateTimeInterval",
        desc="Interval defining the window of time that this time point is valid (for example, seasonal, only on weekends, not on weekends, only 8:00 to 5:00, etc.).",
        transient=True,
        editor=InstanceEditor(name="_datetimeintervals"))

    def _get_datetimeintervals(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.DateTimeInterval" ]
        else:
            return []

    _datetimeintervals = Property(fget=_get_datetimeintervals)

    ScheduledEvents = List(Instance("CIM.IEC61968.Informative.InfCommon.ScheduledEvent"))

    # Time schedule owning this time point.
    TimeSchedule = Instance("CIM.IEC61968.Common.TimeSchedule",
        desc="Time schedule owning this time point.",
        transient=True,
        opposite="TimePoints",
        editor=InstanceEditor(name="_timeschedules"))

    def _get_timeschedules(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.TimeSchedule" ]
        else:
            return []

    _timeschedules = Property(fget=_get_timeschedules)

    # Status of this time point.
    status = Instance("CIM.IEC61968.Common.Status",
        desc="Status of this time point.",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    # (if interval-based) A point in time relative to scheduled start time in 'TimeSchedule.scheduleInterval.start'.
    relativeTimeInterval = Seconds(desc="(if interval-based) A point in time relative to scheduled start time in 'TimeSchedule.scheduleInterval.start'.")

    # Absolute date and time for this time point. For calendar-based time point, it is typically manually entered, while for interval-based or sequence-based time point it is derived.
    absoluteTime = Date(desc="Absolute date and time for this time point. For calendar-based time point, it is typically manually entered, while for interval-based or sequence-based time point it is derived.")

    # (if sequence-based) Relative sequence number for this time point.
    sequenceNumber = Int(desc="(if sequence-based) Relative sequence number for this time point.")

    #--------------------------------------------------------------------------
    #  Begin "TimePoint" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "relativeTimeInterval", "absoluteTime", "sequenceNumber",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "window", "ScheduledEvents", "TimeSchedule", "status",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.TimePoint",
        title="TimePoint",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TimePoint" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "UserAttribute" class:
#------------------------------------------------------------------------------

class UserAttribute(Element):
    """ Generic name-value pair class, with optional sequence number and units for value; can be used to model parts of information exchange when concrete types are not known in advance.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    PropertySpecification = Instance("CIM.IEC61968.Informative.InfAssets.Specification",
        transient=True,
        opposite="AssetProperites",
        editor=InstanceEditor(name="_specifications"))

    def _get_specifications(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.Specification" ]
        else:
            return []

    _specifications = Property(fget=_get_specifications)

    RatingSpecification = Instance("CIM.IEC61968.Informative.InfAssets.Specification",
        transient=True,
        opposite="Ratings",
        editor=InstanceEditor(name="_specifications"))

    def _get_specifications(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.Specification" ]
        else:
            return []

    _specifications = Property(fget=_get_specifications)

    PropertyAssets = List(Instance("CIM.IEC61968.Assets.Asset"))

    RatingAssets = List(Instance("CIM.IEC61968.Assets.Asset"))

    ErpLedgerEntries = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpLedgerEntry"))

    ProcedureDataSets = List(Instance("CIM.IEC61968.Informative.InfAssets.ProcedureDataSet"))

    # Transaction for which this snapshot has been recorded.
    Transaction = Instance("CIM.IEC61968.PaymentMetering.Transaction",
        desc="Transaction for which this snapshot has been recorded.",
        transient=True,
        opposite="UserAttributes",
        editor=InstanceEditor(name="_transactions"))

    def _get_transactions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.Transaction" ]
        else:
            return []

    _transactions = Property(fget=_get_transactions)

    Procedure = Instance("CIM.IEC61968.Informative.InfAssets.Procedure",
        transient=True,
        opposite="ProcedureValues",
        editor=InstanceEditor(name="_procedures"))

    def _get_procedures(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.Procedure" ]
        else:
            return []

    _procedures = Property(fget=_get_procedures)

    PassThroughBills = List(Instance("CIM.IEC61968.Informative.MarketOperations.PassThroughBill"))

    ErpInvoiceLineItems = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpInvoiceLineItem"))

    BillDeterminants = List(Instance("CIM.IEC61968.Informative.MarketOperations.BillDeterminant"))

    ErpStatementLineItems = List(Instance("CIM.IEC61968.Informative.MarketOperations.MarketStatementLineItem"))

    # Sequence number for this attribute in a list of attributes.
    sequenceNumber = Int(desc="Sequence number for this attribute in a list of attributes.")

    # Value of an attribute, including unit information.
    value = StringQuantity(desc="Value of an attribute, including unit information.")

    # Name of an attribute.
    name = Str(desc="Name of an attribute.")

    #--------------------------------------------------------------------------
    #  Begin "UserAttribute" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "sequenceNumber", "value", "name",
                label="Attributes"),
            VGroup("Parent", "PropertySpecification", "RatingSpecification", "PropertyAssets", "RatingAssets", "ErpLedgerEntries", "ProcedureDataSets", "Transaction", "Procedure", "PassThroughBills", "ErpInvoiceLineItems", "BillDeterminants", "ErpStatementLineItems",
                label="References", columns=1),
            dock="tab"),
        id="CIM.IEC61968.Common.UserAttribute",
        title="UserAttribute",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "UserAttribute" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Status" class:
#------------------------------------------------------------------------------

class Status(Element):
    """ Current status information relevant to an entity.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Date and time for which status 'value' applies.
    dateTime = Date(desc="Date and time for which status 'value' applies.")

    # Reason code or explanation for why an object went to the current status 'value'.
    reason = Str(desc="Reason code or explanation for why an object went to the current status 'value'.")

    # Status value at 'dateTime'; prior status changes may have been kept in instances of ActivityRecords associated with the object to which this Status applies.
    value = Str(desc="Status value at 'dateTime'; prior status changes may have been kept in instances of ActivityRecords associated with the object to which this Status applies.")

    # Pertinent information regarding the current 'value', as free form text.
    remark = Str(desc="Pertinent information regarding the current 'value', as free form text.")

    #--------------------------------------------------------------------------
    #  Begin "Status" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "dateTime", "reason", "value", "remark",
                label="Attributes"),
            VGroup("Parent",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.Status",
        title="Status",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Status" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "StreetDetail" class:
#------------------------------------------------------------------------------

class StreetDetail(Element):
    """ Street details, in the context of address.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # (if applicable) In certain cases the physical location of the place of interest does not have a direct point of entry from the street, but may be located inside a larger structure such as a building, complex, office block, apartment, etc.
    buildingName = Str(desc="(if applicable) In certain cases the physical location of the place of interest does not have a direct point of entry from the street, but may be located inside a larger structure such as a building, complex, office block, apartment, etc.")

    # Type of street. Examples include: street, circle, boulevard, avenue, road, drive, etc.
    type = Str(desc="Type of street. Examples include: street, circle, boulevard, avenue, road, drive, etc.")

    # Name of the street.
    name = Str(desc="Name of the street.")

    # True if this street is within the legal geographical boundaries of the specified town (default).
    withinTownLimits = Bool(desc="True if this street is within the legal geographical boundaries of the specified town (default).")

    # Designator of the specific location on the street.
    number = Str(desc="Designator of the specific location on the street.")

    # Suffix to the street name. For example: North, South, East, West.
    suffix = Str(desc="Suffix to the street name. For example: North, South, East, West.")

    # Prefix to the street name. For example: North, South, East, West.
    prefix = Str(desc="Prefix to the street name. For example: North, South, East, West.")

    # (if applicable) Utilities often make use of external reference systems, such as those of the town-planner's department or surveyor general's mapping system, that allocate global reference codes to streets.
    code = Str(desc="(if applicable) Utilities often make use of external reference systems, such as those of the town-planner's department or surveyor general's mapping system, that allocate global reference codes to streets.")

    # Additional address information, for example a mailstop.
    addressGeneral = Str(desc="Additional address information, for example a mailstop.")

    # Number of the apartment or suite.
    suiteNumber = Str(desc="Number of the apartment or suite.")

    #--------------------------------------------------------------------------
    #  Begin "StreetDetail" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "buildingName", "type", "name", "withinTownLimits", "number", "suffix", "prefix", "code", "addressGeneral", "suiteNumber",
                label="Attributes"),
            VGroup("Parent",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Common.StreetDetail",
        title="StreetDetail",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StreetDetail" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Agreement" class:
#------------------------------------------------------------------------------

class Agreement(Document):
    """ Formal agreement between two parties defining the terms and conditions for a set of services. The specifics of the services are, in turn, defined via one or more service agreements.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Date and time interval this agreement is valid (from going into effect to termination).
    validityInterval = Instance("CIM.IEC61968.Common.DateTimeInterval",
        desc="Date and time interval this agreement is valid (from going into effect to termination).",
        transient=True,
        editor=InstanceEditor(name="_datetimeintervals"))

    def _get_datetimeintervals(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.DateTimeInterval" ]
        else:
            return []

    _datetimeintervals = Property(fget=_get_datetimeintervals)

    # Date this agreement was consummated among associated persons and/or organisations.
    signDate = AbsoluteDate(desc="Date this agreement was consummated among associated persons and/or organisations.")

    #--------------------------------------------------------------------------
    #  Begin "Agreement" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "signDate",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "validityInterval",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Common.Agreement",
        title="Agreement",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Agreement" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TimeSchedule" class:
#------------------------------------------------------------------------------

class TimeSchedule(Document):
    """ Description of anything that changes through time. Time schedule is used to perform a single-valued function of time. Use inherited 'category' attribute to give additional information on this schedule, such as: periodic (hourly, daily, weekly, monthly, etc.), day of the month, by date, calendar (specific times and dates).
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Sequence of time points belonging to this time schedule.
    TimePoints = List(Instance("CIM.IEC61968.Common.TimePoint"),
        desc="Sequence of time points belonging to this time schedule.")

    # Schedule date and time interval.
    scheduleInterval = Instance("CIM.IEC61968.Common.DateTimeInterval",
        desc="Schedule date and time interval.",
        transient=True,
        editor=InstanceEditor(name="_datetimeintervals"))

    def _get_datetimeintervals(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.DateTimeInterval" ]
        else:
            return []

    _datetimeintervals = Property(fget=_get_datetimeintervals)

    # True if this schedule is deactivated (disabled).
    disabled = Bool(desc="True if this schedule is deactivated (disabled).")

    # The offset from midnight (i.e., 0 h, 0 min, 0 s) for the periodic time points to begin. For example, for an interval meter that is set up for five minute intervals ('recurrencePeriod'=300=5 min), setting 'offset'=120=2 min would result in scheduled events to read the meter executing at 2 min, 7 min, 12 min, 17 min, 22 min, 27 min, 32 min, 37 min, 42 min, 47 min, 52 min, and 57 min past each hour.
    offset = Seconds(desc="The offset from midnight (i.e., 0 h, 0 min, 0 s) for the periodic time points to begin. For example, for an interval meter that is set up for five minute intervals ('recurrencePeriod'=300=5 min), setting 'offset'=120=2 min would result in scheduled events to read the meter executing at 2 min, 7 min, 12 min, 17 min, 22 min, 27 min, 32 min, 37 min, 42 min, 47 min, 52 min, and 57 min past each hour.")

    # Interval at which the scheduled action repeats (e.g., first Monday of every month, last day of the month, etc.).
    recurrencePattern = Str(desc="Interval at which the scheduled action repeats (e.g., first Monday of every month, last day of the month, etc.).")

    # Duration between time points, from the beginning of one period to the beginning of the next period. Note that a device like a meter may have multiple interval periods (e.g., 1 min, 5 min, 15 min, 30 min, or 60 min).
    recurrencePeriod = Seconds(desc="Duration between time points, from the beginning of one period to the beginning of the next period. Note that a device like a meter may have multiple interval periods (e.g., 1 min, 5 min, 15 min, 30 min, or 60 min).")

    #--------------------------------------------------------------------------
    #  Begin "TimeSchedule" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "disabled", "offset", "recurrencePattern", "recurrencePeriod",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "TimePoints", "scheduleInterval",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Common.TimeSchedule",
        title="TimeSchedule",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TimeSchedule" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "GeoLocation" class:
#------------------------------------------------------------------------------

class GeoLocation(Location):
    """ Geographical location.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All power system resources at this geographical location.
    PowerSystemResources = List(Instance("CIM.IEC61970.Core.PowerSystemResource"),
        desc="All power system resources at this geographical location.")

    #--------------------------------------------------------------------------
    #  Begin "GeoLocation" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "corporateCode", "direction", "isPolygon", "category", "geoInfoReference",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "DocumentRoles", "ErpPersonRoles", "ElectronicAddresses", "ChangeItems", "Routes", "PositionPoints", "GmlSelectors", "mainAddress", "FromLocationRoles", "status", "ToLocationRoles", "TelephoneNumbers", "secondaryAddress", "LandProperties", "Measurements", "ErpOrganisationRoles", "DimensionsInfo", "AssetRoles", "Crews", "RedLines", "GmlObservatins", "Hazards", "ActivityRecords", "PowerSystemResources",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Common.GeoLocation",
        title="GeoLocation",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "GeoLocation" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The domain package is a data dictionary of quantities and units that define datatypes for attributes (properties) that may be used by any class in any other package.  This package contains the definition of primitive datatypes, including units of measure and permissible values. Each datatype contains a value attribute and an optional unit of measure, which is specified as a static variable initialized to the textual description of the unit of measure. The value of the 'units' string may be country or customer specific. Typical values are given. Permissible values for enumerations are listed in the documentation for the attribute using UML constraint syntax inside curly braces. Lengths of variable strings are listed in the descriptive text where required.The domain package is a data dictionary of quantities and units that define datatypes for attributes (properties) that may be used by any class in any other package.  This package contains the definition of primitive datatypes, including units of measure and permissible values. Each datatype contains a value attribute and an optional unit of measure, which is specified as a static variable initialized to the textual description of the unit of measure. The value of the 'units' string may be country or customer specific. Typical values are given. Permissible values for enumerations are listed in the documentation for the attribute using UML constraint syntax inside curly braces. Lengths of variable strings are listed in the descriptive text where required.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------




from enthought.traits.api import Float
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------
Resistance = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatResistance (real part of impedance).Resistance (real part of impedance).")
ApparentPower = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatProduct of the RMS value of the voltage and the RMS value of the currentProduct of the RMS value of the voltage and the RMS value of the current")
AngleRadians = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatPhase angle in radiansPhase angle in radians")
ActivePower = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatProduct of RMS value of the voltage and the RMS value of the in-phase component of the currentProduct of RMS value of the voltage and the RMS value of the in-phase component of the current")
Susceptance = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatImaginary part of admittance.Imaginary part of admittance.")
CurrentFlow = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatElectrical current (positive flow is out of the ConductingEquipment into the ConnectivityNode)Electrical current (positive flow is out of the ConductingEquipment into the ConnectivityNode)")
Conductance = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatFactor by which voltage must be multiplied to give corresponding power lost from a circuit. Real part of admittance.Factor by which voltage must be multiplied to give corresponding power lost from a circuit. Real part of admittance.")
Length = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatUnit of length.Unit of length.")
Temperature = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatValue of temperature in degrees Celsius.Value of temperature in degrees Celsius.")
Reactance = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatReactance (imaginary part of impedance), at rated frequency.Reactance (imaginary part of impedance), at rated frequency.")
KWActivePower = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatActive power in kilowatts.Active power in kilowatts.")
Seconds = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatTime, in secondsTime, in seconds")
Voltage = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatElectrical voltage.Electrical voltage.")
ReactivePower = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatProduct of RMS value of the voltage and the RMS value of the quadrature component of the current.Product of RMS value of the voltage and the RMS value of the quadrature component of the current.")
AngleDegrees = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatMeasurement of angle in degreesMeasurement of angle in degrees")
PerCent = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatNormally 0 - 100 on a defined baseNormally 0 - 100 on a defined base")
Impedance = Float(0, desc="http://www.w3.org/2001/XMLSchema#floatRatio of voltage to current.Ratio of voltage to current.")




# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" An extension to the Core and Wires packages that models information on the current and planned network configuration. These entities are optional within typical network applications.
"""

from cim.iec61970.core import IdentifiedObject
from cim.iec61970.core import IrregularIntervalSchedule

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim.outage"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#Outage"

class ClearanceTag(IdentifiedObject):
    """ A clearance tag that is used to authorize and schedule work on conducting equipment in the field. Tagged equipment is not available for commercial service.
    """
    # <<< clearance_tag
    # @generated
    def __init__(self, authority_name='', work_end_time='', phase_check_req_flag=False, deenergize_req_flag=False, tag_issue_time='', work_description='', work_start_time='', ground_req_flag=False, conducting_equipment=None, safety_document=None, clearance_tag_type=None, **kw_args):
        """ Initialises a new 'ClearanceTag' instance.
        """
        # The name of the person who is authorized to issue the tag 
        self.authority_name = authority_name

        # The time at which the clearance tag is scheduled to be removed 
        self.work_end_time = work_end_time

        # Set true if equipment phasing must be checked 
        self.phase_check_req_flag = phase_check_req_flag

        # Set true if equipment must be deenergized 
        self.deenergize_req_flag = deenergize_req_flag

        # The time at which the clearance tag was issued 
        self.tag_issue_time = tag_issue_time

        # Description of the work to be performed 
        self.work_description = work_description

        # The time at which the clearance tag is scheduled to be set. 
        self.work_start_time = work_start_time

        # Set true if equipment must be grounded 
        self.ground_req_flag = ground_req_flag


        self._conducting_equipment = None
        self.conducting_equipment = conducting_equipment

        self._safety_document = None
        self.safety_document = safety_document

        self._clearance_tag_type = None
        self.clearance_tag_type = clearance_tag_type


        super(ClearanceTag, self).__init__(**kw_args)
    # >>> clearance_tag

    # <<< conducting_equipment
    # @generated
    def get_conducting_equipment(self):
        """ Conducting equipment may have multiple clearance tags for authorized field work
        """
        return self._conducting_equipment

    def set_conducting_equipment(self, value):
        if self._conducting_equipment is not None:
            filtered = [x for x in self.conducting_equipment.clearance_tags if x != self]
            self._conducting_equipment._clearance_tags = filtered

        self._conducting_equipment = value
        if self._conducting_equipment is not None:
            self._conducting_equipment._clearance_tags.append(self)

    conducting_equipment = property(get_conducting_equipment, set_conducting_equipment)
    # >>> conducting_equipment

    # <<< safety_document
    # @generated
    def get_safety_document(self):
        """ 
        """
        return self._safety_document

    def set_safety_document(self, value):
        if self._safety_document is not None:
            filtered = [x for x in self.safety_document.clearance_tags if x != self]
            self._safety_document._clearance_tags = filtered

        self._safety_document = value
        if self._safety_document is not None:
            self._safety_document._clearance_tags.append(self)

    safety_document = property(get_safety_document, set_safety_document)
    # >>> safety_document

    # <<< clearance_tag_type
    # @generated
    def get_clearance_tag_type(self):
        """ The type of tag, depending on the purpose of the work to be performed and/or the type of supervisory control allowed.
        """
        return self._clearance_tag_type

    def set_clearance_tag_type(self, value):
        if self._clearance_tag_type is not None:
            filtered = [x for x in self.clearance_tag_type.clearance_tags if x != self]
            self._clearance_tag_type._clearance_tags = filtered

        self._clearance_tag_type = value
        if self._clearance_tag_type is not None:
            self._clearance_tag_type._clearance_tags.append(self)

    clearance_tag_type = property(get_clearance_tag_type, set_clearance_tag_type)
    # >>> clearance_tag_type


    def __str__(self):
        """ Returns a string representation of the ClearanceTag.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< clearance_tag.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the ClearanceTag.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "ClearanceTag", self.uri)
        if format:
            indent += ' ' * depth

        if self.conducting_equipment is not None:
            s += '%s<%s:ClearanceTag.conducting_equipment rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.conducting_equipment.uri)
        if self.safety_document is not None:
            s += '%s<%s:ClearanceTag.safety_document rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.safety_document.uri)
        if self.clearance_tag_type is not None:
            s += '%s<%s:ClearanceTag.clearance_tag_type rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.clearance_tag_type.uri)
        s += '%s<%s:ClearanceTag.authority_name>%s</%s:ClearanceTag.authority_name>' % \
            (indent, ns_prefix, self.authority_name, ns_prefix)
        s += '%s<%s:ClearanceTag.work_end_time>%s</%s:ClearanceTag.work_end_time>' % \
            (indent, ns_prefix, self.work_end_time, ns_prefix)
        s += '%s<%s:ClearanceTag.phase_check_req_flag>%s</%s:ClearanceTag.phase_check_req_flag>' % \
            (indent, ns_prefix, self.phase_check_req_flag, ns_prefix)
        s += '%s<%s:ClearanceTag.deenergize_req_flag>%s</%s:ClearanceTag.deenergize_req_flag>' % \
            (indent, ns_prefix, self.deenergize_req_flag, ns_prefix)
        s += '%s<%s:ClearanceTag.tag_issue_time>%s</%s:ClearanceTag.tag_issue_time>' % \
            (indent, ns_prefix, self.tag_issue_time, ns_prefix)
        s += '%s<%s:ClearanceTag.work_description>%s</%s:ClearanceTag.work_description>' % \
            (indent, ns_prefix, self.work_description, ns_prefix)
        s += '%s<%s:ClearanceTag.work_start_time>%s</%s:ClearanceTag.work_start_time>' % \
            (indent, ns_prefix, self.work_start_time, ns_prefix)
        s += '%s<%s:ClearanceTag.ground_req_flag>%s</%s:ClearanceTag.ground_req_flag>' % \
            (indent, ns_prefix, self.ground_req_flag, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "ClearanceTag")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> clearance_tag.serialize


class ClearanceTagType(IdentifiedObject):
    """ Type of ClearanceTag. Could indicate the type of work to be performed and/or the type of supervisory control.
    """
    # <<< clearance_tag_type
    # @generated
    def __init__(self, clearance_tags=None, **kw_args):
        """ Initialises a new 'ClearanceTagType' instance.
        """

        self._clearance_tags = []
        if clearance_tags is not None:
            self.clearance_tags = clearance_tags
        else:
            self.clearance_tags = []


        super(ClearanceTagType, self).__init__(**kw_args)
    # >>> clearance_tag_type

    # <<< clearance_tags
    # @generated
    def get_clearance_tags(self):
        """ The ClearanceTags currently being defined for this type.
        """
        return self._clearance_tags

    def set_clearance_tags(self, value):
        for x in self._clearance_tags:
            x._clearance_tag_type = None
        for y in value:
            y._clearance_tag_type = self
        self._clearance_tags = value

    clearance_tags = property(get_clearance_tags, set_clearance_tags)

    def add_clearance_tags(self, *clearance_tags):
        for obj in clearance_tags:
            obj._clearance_tag_type = self
            self._clearance_tags.append(obj)

    def remove_clearance_tags(self, *clearance_tags):
        for obj in clearance_tags:
            obj._clearance_tag_type = None
            self._clearance_tags.remove(obj)
    # >>> clearance_tags


    def __str__(self):
        """ Returns a string representation of the ClearanceTagType.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< clearance_tag_type.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the ClearanceTagType.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "ClearanceTagType", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.clearance_tags:
            s += '%s<%s:ClearanceTagType.clearance_tags rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "ClearanceTagType")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> clearance_tag_type.serialize


class OutageSchedule(IrregularIntervalSchedule):
    """ The period of time that a piece of equipment is out of service, for example, for maintenance or testing; including the equipment's active power rating while under maintenance. The X-axis represents absolute time and the Y-axis represents the equipment's available rating while out of service.
    """
    # <<< outage_schedule
    # @generated
    def __init__(self, power_system_resource=None, switching_operations=None, planned_outage=None, **kw_args):
        """ Initialises a new 'OutageSchedule' instance.
        """

        self._power_system_resource = None
        self.power_system_resource = power_system_resource

        self._switching_operations = []
        if switching_operations is not None:
            self.switching_operations = switching_operations
        else:
            self.switching_operations = []

        self._planned_outage = None
        self.planned_outage = planned_outage


        super(OutageSchedule, self).__init__(**kw_args)
    # >>> outage_schedule

    # <<< power_system_resource
    # @generated
    def get_power_system_resource(self):
        """ A power system resource may have an outage schedule
        """
        return self._power_system_resource

    def set_power_system_resource(self, value):
        if self._power_system_resource is not None:
            self._power_system_resource._outage_schedule = None

        self._power_system_resource = value
        if self._power_system_resource is not None:
            self._power_system_resource._outage_schedule = self

    power_system_resource = property(get_power_system_resource, set_power_system_resource)
    # >>> power_system_resource

    # <<< switching_operations
    # @generated
    def get_switching_operations(self):
        """ An OutageSchedule may operate many switches.
        """
        return self._switching_operations

    def set_switching_operations(self, value):
        for x in self._switching_operations:
            x._outage_schedule = None
        for y in value:
            y._outage_schedule = self
        self._switching_operations = value

    switching_operations = property(get_switching_operations, set_switching_operations)

    def add_switching_operations(self, *switching_operations):
        for obj in switching_operations:
            obj._outage_schedule = self
            self._switching_operations.append(obj)

    def remove_switching_operations(self, *switching_operations):
        for obj in switching_operations:
            obj._outage_schedule = None
            self._switching_operations.remove(obj)
    # >>> switching_operations

    # <<< planned_outage
    # @generated
    def get_planned_outage(self):
        """ 
        """
        return self._planned_outage

    def set_planned_outage(self, value):
        if self._planned_outage is not None:
            filtered = [x for x in self.planned_outage.outage_schedules if x != self]
            self._planned_outage._outage_schedules = filtered

        self._planned_outage = value
        if self._planned_outage is not None:
            self._planned_outage._outage_schedules.append(self)

    planned_outage = property(get_planned_outage, set_planned_outage)
    # >>> planned_outage


    def __str__(self):
        """ Returns a string representation of the OutageSchedule.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< outage_schedule.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the OutageSchedule.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "OutageSchedule", self.uri)
        if format:
            indent += ' ' * depth

        if self.power_system_resource is not None:
            s += '%s<%s:OutageSchedule.power_system_resource rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.power_system_resource.uri)
        for obj in self.switching_operations:
            s += '%s<%s:OutageSchedule.switching_operations rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.planned_outage is not None:
            s += '%s<%s:OutageSchedule.planned_outage rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.planned_outage.uri)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        s += '%s<%s:BasicIntervalSchedule.value2_multiplier>%s</%s:BasicIntervalSchedule.value2_multiplier>' % \
            (indent, ns_prefix, self.value2_multiplier, ns_prefix)
        s += '%s<%s:BasicIntervalSchedule.value1_unit>%s</%s:BasicIntervalSchedule.value1_unit>' % \
            (indent, ns_prefix, self.value1_unit, ns_prefix)
        s += '%s<%s:BasicIntervalSchedule.value2_unit>%s</%s:BasicIntervalSchedule.value2_unit>' % \
            (indent, ns_prefix, self.value2_unit, ns_prefix)
        s += '%s<%s:BasicIntervalSchedule.value1_multiplier>%s</%s:BasicIntervalSchedule.value1_multiplier>' % \
            (indent, ns_prefix, self.value1_multiplier, ns_prefix)
        s += '%s<%s:BasicIntervalSchedule.start_time>%s</%s:BasicIntervalSchedule.start_time>' % \
            (indent, ns_prefix, self.start_time, ns_prefix)
        for obj in self.time_points:
            s += '%s<%s:IrregularIntervalSchedule.time_points rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "OutageSchedule")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> outage_schedule.serialize


class SwitchingOperation(IdentifiedObject):
    """ A SwitchingOperation is used to define individual switch operations for an OutageSchedule. This OutageSchedule may be associated with another item of Substation such as a Transformer, Line, or Generator; or with the Switch itself as a PowerSystemResource. A Switch may be referenced by many OutageSchedules.
    """
    # <<< switching_operation
    # @generated
    def __init__(self, new_state='open', operation_time='', outage_schedule=None, switches=None, **kw_args):
        """ Initialises a new 'SwitchingOperation' instance.
        """
        # The switch position that shall result from this SwitchingOperation Values are: "open", "close"
        self.new_state = 'open'

        # Time of operation in same units as OutageSchedule.xAxixUnits. 
        self.operation_time = operation_time


        self._outage_schedule = None
        self.outage_schedule = outage_schedule

        self._switches = []
        if switches is not None:
            self.switches = switches
        else:
            self.switches = []


        super(SwitchingOperation, self).__init__(**kw_args)
    # >>> switching_operation

    # <<< outage_schedule
    # @generated
    def get_outage_schedule(self):
        """ An OutageSchedule may operate many switches.
        """
        return self._outage_schedule

    def set_outage_schedule(self, value):
        if self._outage_schedule is not None:
            filtered = [x for x in self.outage_schedule.switching_operations if x != self]
            self._outage_schedule._switching_operations = filtered

        self._outage_schedule = value
        if self._outage_schedule is not None:
            self._outage_schedule._switching_operations.append(self)

    outage_schedule = property(get_outage_schedule, set_outage_schedule)
    # >>> outage_schedule

    # <<< switches
    # @generated
    def get_switches(self):
        """ A switch may be operated by many schedules.
        """
        return self._switches

    def set_switches(self, value):
        for p in self._switches:
            filtered = [q for q in p.switching_operations if q != self]
            self._switches._switching_operations = filtered
        for r in value:
            if self not in r._switching_operations:
                r._switching_operations.append(self)
        self._switches = value

    switches = property(get_switches, set_switches)

    def add_switches(self, *switches):
        for obj in switches:
            if self not in obj._switching_operations:
                obj._switching_operations.append(self)
            self._switches.append(obj)

    def remove_switches(self, *switches):
        for obj in switches:
            if self in obj._switching_operations:
                obj._switching_operations.remove(self)
            self._switches.remove(obj)
    # >>> switches


    def __str__(self):
        """ Returns a string representation of the SwitchingOperation.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< switching_operation.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SwitchingOperation.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SwitchingOperation", self.uri)
        if format:
            indent += ' ' * depth

        if self.outage_schedule is not None:
            s += '%s<%s:SwitchingOperation.outage_schedule rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.outage_schedule.uri)
        for obj in self.switches:
            s += '%s<%s:SwitchingOperation.switches rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:SwitchingOperation.new_state>%s</%s:SwitchingOperation.new_state>' % \
            (indent, ns_prefix, self.new_state, ns_prefix)
        s += '%s<%s:SwitchingOperation.operation_time>%s</%s:SwitchingOperation.operation_time>' % \
            (indent, ns_prefix, self.operation_time, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SwitchingOperation")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> switching_operation.serialize


# <<< outage
# @generated
# >>> outage



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" An extension to the Core and Wires packages that models information on the current and planned network configuration. These entities are optional within typical network applications.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61970.Core import IdentifiedObject
from CIM.IEC61970.Core import IrregularIntervalSchedule



from enthought.traits.api import Instance, List, Property, Enum, Str, Date, Bool
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------

# Possible states for a switch.
SwitchState = Enum("open", "close", desc="Possible states for a switch.")

#------------------------------------------------------------------------------
#  "ClearanceTag" class:
#------------------------------------------------------------------------------

class ClearanceTag(IdentifiedObject):
    """ A clearance tag that is used to authorize and schedule work on conducting equipment in the field. Tagged equipment is not available for commercial service.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Conducting equipment may have multiple clearance tags for authorized field work
    ConductingEquipment = Instance("CIM.IEC61970.Core.ConductingEquipment",
        desc="Conducting equipment may have multiple clearance tags for authorized field work",
        transient=True,
        opposite="ClearanceTags",
        editor=InstanceEditor(name="_conductingequipments"))

    def _get_conductingequipments(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Core.ConductingEquipment" ]
        else:
            return []

    _conductingequipments = Property(fget=_get_conductingequipments)

    SafetyDocument = Instance("CIM.IEC61968.Informative.InfOperations.SafetyDocument",
        transient=True,
        opposite="ClearanceTags",
        editor=InstanceEditor(name="_safetydocuments"))

    def _get_safetydocuments(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfOperations.SafetyDocument" ]
        else:
            return []

    _safetydocuments = Property(fget=_get_safetydocuments)

    # The type of tag, depending on the purpose of the work to be performed and/or the type of supervisory control allowed.
    ClearanceTagType = Instance("CIM.IEC61970.Outage.ClearanceTagType",
        desc="The type of tag, depending on the purpose of the work to be performed and/or the type of supervisory control allowed.",
        transient=True,
        opposite="ClearanceTags",
        editor=InstanceEditor(name="_clearancetagtypes"))

    def _get_clearancetagtypes(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Outage.ClearanceTagType" ]
        else:
            return []

    _clearancetagtypes = Property(fget=_get_clearancetagtypes)

    # The name of the person who is authorized to issue the tag
    authorityName = Str(desc="The name of the person who is authorized to issue the tag")

    # The time at which the clearance tag is scheduled to be removed
    workEndTime = Date(desc="The time at which the clearance tag is scheduled to be removed")

    # Set true if equipment phasing must be checked
    phaseCheckReqFlag = Bool(desc="Set true if equipment phasing must be checked")

    # Set true if equipment must be deenergized
    deenergizeReqFlag = Bool(desc="Set true if equipment must be deenergized")

    # The time at which the clearance tag was issued
    tagIssueTime = Date(desc="The time at which the clearance tag was issued")

    # Description of the work to be performed
    workDescription = Str(desc="Description of the work to be performed")

    # The time at which the clearance tag is scheduled to be set.
    workStartTime = Date(desc="The time at which the clearance tag is scheduled to be set.")

    # Set true if equipment must be grounded
    groundReqFlag = Bool(desc="Set true if equipment must be grounded")

    #--------------------------------------------------------------------------
    #  Begin "ClearanceTag" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "authorityName", "workEndTime", "phaseCheckReqFlag", "deenergizeReqFlag", "tagIssueTime", "workDescription", "workStartTime", "groundReqFlag",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ConductingEquipment", "SafetyDocument", "ClearanceTagType",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.Outage.ClearanceTag",
        title="ClearanceTag",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ClearanceTag" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ClearanceTagType" class:
#------------------------------------------------------------------------------

class ClearanceTagType(IdentifiedObject):
    """ Type of ClearanceTag. Could indicate the type of work to be performed and/or the type of supervisory control.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The ClearanceTags currently being defined for this type.
    ClearanceTags = List(Instance("CIM.IEC61970.Outage.ClearanceTag"),
        desc="The ClearanceTags currently being defined for this type.")

    #--------------------------------------------------------------------------
    #  Begin "ClearanceTagType" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "ClearanceTags",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.Outage.ClearanceTagType",
        title="ClearanceTagType",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ClearanceTagType" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OutageSchedule" class:
#------------------------------------------------------------------------------

class OutageSchedule(IrregularIntervalSchedule):
    """ The period of time that a piece of equipment is out of service, for example, for maintenance or testing; including the equipment's active power rating while under maintenance. The X-axis represents absolute time and the Y-axis represents the equipment's available rating while out of service.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # A power system resource may have an outage schedule
    PowerSystemResource = Instance("CIM.IEC61970.Core.PowerSystemResource",
        desc="A power system resource may have an outage schedule",
        transient=True,
        opposite="OutageSchedule",
        editor=InstanceEditor(name="_powersystemresources"))

    def _get_powersystemresources(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Core.PowerSystemResource" ]
        else:
            return []

    _powersystemresources = Property(fget=_get_powersystemresources)

    # An OutageSchedule may operate many switches.
    SwitchingOperations = List(Instance("CIM.IEC61970.Outage.SwitchingOperation"),
        desc="An OutageSchedule may operate many switches.")

    PlannedOutage = Instance("CIM.IEC61968.Informative.InfOperations.PlannedOutage",
        transient=True,
        opposite="OutageSchedules",
        editor=InstanceEditor(name="_plannedoutages"))

    def _get_plannedoutages(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfOperations.PlannedOutage" ]
        else:
            return []

    _plannedoutages = Property(fget=_get_plannedoutages)

    #--------------------------------------------------------------------------
    #  Begin "OutageSchedule" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "value2Multiplier", "value1Unit", "value2Unit", "value1Multiplier", "startTime",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "TimePoints", "PowerSystemResource", "SwitchingOperations", "PlannedOutage",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.Outage.OutageSchedule",
        title="OutageSchedule",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OutageSchedule" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SwitchingOperation" class:
#------------------------------------------------------------------------------

class SwitchingOperation(IdentifiedObject):
    """ A SwitchingOperation is used to define individual switch operations for an OutageSchedule. This OutageSchedule may be associated with another item of Substation such as a Transformer, Line, or Generator; or with the Switch itself as a PowerSystemResource. A Switch may be referenced by many OutageSchedules.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # An OutageSchedule may operate many switches.
    OutageSchedule = Instance("CIM.IEC61970.Outage.OutageSchedule",
        desc="An OutageSchedule may operate many switches.",
        transient=True,
        opposite="SwitchingOperations",
        editor=InstanceEditor(name="_outageschedules"))

    def _get_outageschedules(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Outage.OutageSchedule" ]
        else:
            return []

    _outageschedules = Property(fget=_get_outageschedules)

    # A switch may be operated by many schedules.
    Switches = List(Instance("CIM.IEC61970.Wires.Switch"),
        desc="A switch may be operated by many schedules.")

    # The switch position that shall result from this SwitchingOperation
    newState = SwitchState(desc="The switch position that shall result from this SwitchingOperation")

    # Time of operation in same units as OutageSchedule.xAxixUnits.
    operationTime = Date(desc="Time of operation in same units as OutageSchedule.xAxixUnits.")

    #--------------------------------------------------------------------------
    #  Begin "SwitchingOperation" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "newState", "operationTime",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OutageSchedule", "Switches",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.Outage.SwitchingOperation",
        title="SwitchingOperation",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SwitchingOperation" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------
